HookFuncs={
    "kernel32.dll_GetModuleHandleA" : 0,
    "kernel32.dll_LoadLibraryA" : 1,
    "kernel32.dll_GetProcAddress" : 2,
    "ntdll.dll_ZwOpenThread" : 3,
    "kernel32.dll_GetUserDefaultUILanguage" : 4,
    "ntdll.dll_RtlAllocateHeap" : 5,
    "kernel32.dll_GetCurrentDirectoryW" : 6,
    "kernel32.dll_GetModuleFileNameW" : 7,
    "kernel32.dll_SetCurrentDirectoryW" : 8,
    "kernel32.dll_GetCommandLineA" : 9,
    "ntdll.dll_ZwQueryInformationProcess" : 10,
    "ntdll.dll_ZwAllocateVirtualMemory" : 11,
    "ntdll.dll_ZwGetContextThread" : 12,
    "kernel32.dll_VirtualFree" : 13,
    "ntdll.dll_ZwSetInformationThread" : 14,
    "advapi32.dll_OpenThreadToken" : 15,
    "advapi32.dll_OpenProcessToken" : 16,
    "ntdll.dll_ZwQueryInformationToken" : 17,
    "ntdll.dll_ZwSetInformationProcess" : 18,
    "ntdll.dll_ZwClose" : 19,
    "ntdll.dll_RtlFreeHeap" : 20,
    "ntdll.dll_ZwOpenThreadTokenEx" : 21,
    "ntdll.dll_ZwOpenProcessTokenEx" : 22,
    "ntdll.dll_ZwDuplicateToken" : 23,
    "ntdll.dll_ZwAccessCheck" : 24,
    "kernel32.dll_VirtualProtect" : 25,
    "win32u.dll_NtUserGetForegroundWindow" : 26,
    "user32.dll_GetWindowTextA" : 27,
    "ntdll.dll_ZwQuerySystemInformation" : 28,
    "kernelbase.dll_RegOpenKeyExA" : 29,
    "advapi32.dll_RegQueryValueExA" : 30,
    "advapi32.dll_RegCloseKey" : 31,
    "kernelbase.dll_VirtualProtect" : 32,
}