REFLECTOR = {
        "api-ms-win-core-sysinfo-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-sysinfo-l1-2-0.dll" : "kernelbase.dll",
        "api-ms-win-core-sysinfo-l1-2-1.dll" : "kernelbase.dll",
        "api-ms-win-core-libraryloader-l1-2-0.dll" : "kernelbase.dll",
        "api-ms-win-core-libraryloader-l1-2-1.dll" : "kernelbase.dll",
        "api-ms-win-core-errorhandling-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-libraryloader-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-threadpool-legacy-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-threadpool-private-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-timezone-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-crt-math-l1-1-0.dll" : "ucrtbase.dll",
        "api-ms-win-crt-time-l1-1-0.dll" : "ucrtbase.dll",
        "api-ms-win-crt-runtime-l1-1-0.dll" : "ucrtbase.dll",
        "api-ms-win-crt-heap-l1-1-0.dll" : "ucrtbase.dll",
        "api-ms-win-crt-heap-l1-1-0.dll" : "ucrtbase.dll",
        "api-ms-win-crt-utility-l1-1-0.dll": "ucrtbase.dll",
        "api-ms-win-crt-stdio-l1-1-0.dll" : "ucrtbase.dll",
        "api-ms-win-crt-locale-l1-1-0.dll" : "ucrtbase.dll",
        "api-ms-win-core-synch-l1-1-0.dll" : "ntdll.dll",
        "api-ms-win-core-heap-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-processthreads-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-processthreads-l1-1-1.dll" : "kernelbase.dll",
        "api-ms-win-core-processthreads-l1-1-2.dll" : "kernelbase.dll",
        "api-ms-win-core-xstate-l2-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-wow64-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-realtime-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-io-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-io-l1-1-1.dll" : "kernelbase.dll",
        "api-ms-win-core-memory-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-memory-l1-1-1.dll" : "kernelbase.dll",
        "api-ms-win-core-memory-l1-1-2.dll" : "kernelbase.dll",
        "api-ms-win-core-file-l1-1-0.dll" : "kernel32.dll",
        "api-ms-win-core-file-l1-2-0.dll" : "kernel32.dll",
        "api-ms-win-core-file-l1-2-1.dll" : "kernel32.dll",
        "api-ms-win-core-file-l2-1-0.dll" : "kernel32.dll",
        "api-ms-win-core-file-l2-1-1.dll" : "kernel32.dll",
        "api-ms-win-core-rtlsupport-l1-1-0.dll" : "kernel32.dll",
        "api-ms-win-core-console-l1-1-0.dll" : "kernel32.dll",
        "api-ms-win-eventing-provider-l1-1-0.dll" : "advapi32.dll",
        "api-ms-win-core-console-l1-1-0.dll" : "kernel32.dll",
        "api-ms-win-core-datetime-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-datetime-l1-1-1.dll" : "kernelbase.dll",
        "api-ms-win-core-datetime-l1-1-2.dll" : "kernelbase.dll",
        "api-ms-win-core-debug-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-debug-l1-1-1.dll" : "kernelbase.dll",
        "api-ms-win-core-fibers-l1-1-0.dll" : "kernel32.dll",
        "api-ms-win-core-handle-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-localization-l1-2-0.dll" : "kernel32.dll",
        "api-ms-win-core-namedpipe-l1-1-0.dll" : "kernel32.dll",
        "api-ms-win-core-processenvironment-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-processenvironment-l1-2-0.dll" : "kernelbase.dll",
        "api-ms-win-core-profile-l1-1-0.dll" : "kernel32.dll",
        "api-ms-win-core-string-l1-1-0.dll" : "kernel32.dll",
        "api-ms-win-core-synch-l1-2-0.dll" : "kernel32.dll",
        "api-ms-win-core-util-l1-1-0.dll" : "kernel32.dll",
        "api-ms-win-security-base-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-security-base-l1-2-0.dll" : "kernelbase.dll",
        "api-ms-win-core-registry-l1-1-0.dll" : "kernelbase.dll",
        "api-ms-win-core-registry-l1-1-1.dll" : "kernelbase.dll",
        "api-ms-win-core-registry-l1-1-2.dll" : "kernelbase.dll",
        "api-ms-win-core-com-l1-1-0.dll" : "ole32.dll"
    }